@@ -7,12 +7,10 @@
 
 //! Various storage and representation related types
 
-use crate::append_only_zks::Azks;
 use crate::history_tree_node::{HistoryTreeNode, NodeType};
-use crate::node_state::{HistoryChildState, HistoryNodeState, NodeLabel, NodeStateKey};
+use crate::node_state::{HistoryChildState, HistoryNodeState, NodeStateKey};
 use crate::storage::Storable;
-use crate::ARITY;
-use serde::{Deserialize, Serialize};
+use crate::{Azks, NodeLabel, ARITY};
 use std::convert::TryInto;
 
 /// Various elements that can be stored
@@ -29,21 +27,52 @@ pub enum StorageType {
 }
 
 /// The keys for this key-value store
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
-pub struct AkdLabel(pub String);
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+pub struct AkdLabel(pub Vec<u8>);
 
+impl AkdLabel {
+    /// Build an [`AkdLabel`] struct from an UTF8 string
+    pub fn from_utf8_str(value: &str) -> Self {
+        Self(value.as_bytes().to_vec())
+    }
+}
+
+impl core::ops::Deref for AkdLabel {
+    type Target = [u8];
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
 /// The types of value used in the key-value pairs of a AKD
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
-#[serde(bound = "")]
-pub struct AkdValue(pub String);
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(bound = ""))]
+pub struct AkdValue(pub Vec<u8>);
+
+impl AkdValue {
+    /// Build an [`AkdValue`] struct from an UTF8 string
+    pub fn from_utf8_str(value: &str) -> Self {
+        Self(value.as_bytes().to_vec())
+    }
+}
+
+impl core::ops::Deref for AkdValue {
+    type Target = [u8];
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
 
 /// State for a value at a given version for that key
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
-pub struct ValueStateKey(pub String, pub u64);
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+pub struct ValueStateKey(pub Vec<u8>, pub u64);
 
 /// The state of the value for a given key, starting at a particular epoch.
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
-#[serde(bound = "")]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(bound = ""))]
 pub struct ValueState {
     /// The plaintext value of the user information in the directory
     pub plaintext_val: AkdValue, // This needs to be the plaintext value, to discuss
@@ -65,13 +94,13 @@ impl crate::storage::Storable for ValueState {
     }
 
     fn get_id(&self) -> ValueStateKey {
-        ValueStateKey(self.username.0.clone(), self.epoch)
+        ValueStateKey(self.username.to_vec(), self.epoch)
     }
 
     fn get_full_binary_key_id(key: &ValueStateKey) -> Vec<u8> {
         let mut result = vec![StorageType::ValueState as u8];
-        result.extend_from_slice(&key.1.to_be_bytes());
-        result.extend_from_slice(key.0.as_bytes());
+        result.extend_from_slice(&key.1.to_le_bytes());
+        result.extend_from_slice(&key.0);
 
         result
     }
@@ -81,12 +110,8 @@ impl crate::storage::Storable for ValueState {
             return Err("Not enough bytes to form a proper key".to_string());
         }
         let epoch_bytes: [u8; 8] = bin[1..=8].try_into().expect("Slice with incorrect length");
-        let epoch = u64::from_be_bytes(epoch_bytes);
-        if let Ok(username) = std::str::from_utf8(&bin[9..]) {
-            Ok(ValueStateKey(username.to_string(), epoch))
-        } else {
-            Err("Invalid string format".to_string())
-        }
+        let epoch = u64::from_le_bytes(epoch_bytes);
+        Ok(ValueStateKey(bin[9..].to_vec(), epoch))
     }
 }
 
@@ -109,8 +134,9 @@ impl ValueState {
 }
 
 /// Data associated with a given key. That is all the states at the various epochs
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
-#[serde(bound = "")]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+#[cfg_attr(feature = "serde", serde(bound = ""))]
 pub struct KeyData {
     /// The vector of states of key data for a given AkdLabel
     pub states: Vec<ValueState>,
@@ -135,7 +161,8 @@ pub enum ValueStateRetrievalFlag {
 
 /// This needs to be PUBLIC public, since anyone implementing a data-layer will need
 /// to be able to access this and all the internal types
-#[derive(Serialize, Deserialize, Debug, PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub enum DbRecord {
     /// An Azks
     Azks(Azks),
@@ -159,7 +186,9 @@ impl Clone for DbRecord {
 }
 
 impl DbRecord {
-    pub(crate) fn get_full_binary_id(&self) -> Vec<u8> {
+    /// Compte a serialized id from the record's fields. This id is useful to use as key
+    /// in key-value stores.
+    pub fn get_full_binary_id(&self) -> Vec<u8> {
         match &self {
             DbRecord::Azks(azks) => azks.get_full_binary_id(),
             DbRecord::HistoryNodeState(state) => state.get_full_binary_id(),
@@ -168,6 +197,18 @@ impl DbRecord {
         }
     }
 
+    /// Returns the priority in which a record type in a transaction should be committed to storage.
+    /// A smaller value indicates higher priority in being written first.
+    /// An Azks record should always be updated last, so that any concurrent storage readers will
+    /// not see an increase in the current epoch until every other record for the new epoch has
+    /// been written to storage.
+    pub(crate) fn transaction_priority(&self) -> u8 {
+        match &self {
+            DbRecord::Azks(_) => 2,
+            _ => 1,
+        }
+    }
+
     /* Data Layer Builders */
 
     /// Build an azks instance from the properties
@@ -180,11 +221,11 @@ impl DbRecord {
 
     /// Build a history tree node from the properties
     pub fn build_history_tree_node(
-        label_val: u64,
+        label_val: [u8; 32],
         label_len: u32,
         birth_epoch: u64,
         last_epoch: u64,
-        parent_label_val: u64,
+        parent_label_val: [u8; 32],
         parent_label_len: u32,
         node_type: u8,
     ) -> HistoryTreeNode {
@@ -199,10 +240,10 @@ impl DbRecord {
 
     /// Build a history node state from the properties
     pub fn build_history_node_state(
-        value: Vec<u8>,
+        value: [u8; 32],
         child_states: [Option<HistoryChildState>; ARITY],
         label_len: u32,
-        label_val: u64,
+        label_val: [u8; 32],
         epoch: u64,
     ) -> HistoryNodeState {
         HistoryNodeState {
@@ -215,8 +256,8 @@ impl DbRecord {
     /// Build a history child state from the properties
     pub fn build_history_child_state(
         label_len: u32,
-        label_val: u64,
-        hash_val: Vec<u8>,
+        label_val: [u8; 32],
+        hash_val: [u8; 32],
         epoch_version: u64,
     ) -> HistoryChildState {
         HistoryChildState {
@@ -228,11 +269,11 @@ impl DbRecord {
 
     /// Build a user state from the properties
     pub fn build_user_state(
-        username: String,
-        plaintext_val: String,
+        username: Vec<u8>,
+        plaintext_val: Vec<u8>,
         version: u64,
         label_len: u32,
-        label_val: u64,
+        label_val: [u8; 32],
         epoch: u64,
     ) -> ValueState {
         ValueState {
